<?php
if (!defined('_PS_VERSION_'))
  exit;
 
class Comnpay extends PaymentModule
{


	public function __construct()
	{
		$this->name = 'comnpay';
		$this->tab = 'payments_gateways';
		$this->version = '0.9.0';
		$this->author = 'Afone';
		$this->need_instance = 1;
		$this->ps_versions_compliancy = array('min' => '1.4', 'max' => '1.4');
		$this->dependencies = array('blockcart');
		$this->currencies = true;
		$this->currencies_mode = 'checkbox';

		parent::__construct();

		$this->displayName = $this->l('Comnpay');
		$this->description = $this->l('Accepts payments by credit cards with comNpay.');

		$this->confirmUninstall = $this->l('Are you sure you want to uninstall?');

		require(_PS_MODULE_DIR_.$this->name.'/backward_compatibility/backward.php');
	}


	public function install()
	{		
		include_once(_PS_MODULE_DIR_.'/'.$this->name.'/comnpay_install.php');
		$comnpay_install = new ComnpayInstall();
		$comnpay_install->updateConfiguration($this->version);
		$comnpay_install->createOrderState();

		return parent::install() &&
							$this->registerHook('payment') && 
							$this->registerHook('paymentReturn');
	}

	public function uninstall()
	{
		include_once(_PS_MODULE_DIR_.'/'.$this->name.'/comnpay_install.php');
		$comnpay_install = new ComnpayInstall();
		$comnpay_install->deleteConfiguration();

		if (!$this->unregisterHook('payment') || !$this->unregisterHook('paymentReturn') )
		{
			Logger::addLog('Comnpay module: unregisterHook failed', 4);
			return false;
		}

		if (!parent::uninstall())
		{
			Logger::addLog('Comnpay module: uninstall failed', 4);
			return false;
		}

		return true;
	}

	/**
	 * Sur PS 1.4, le hook est hookPayment et non hookDisplayPayment => on fait la redirection
	 */
	public function hookPayment($params)
	{
		return $this->hookDisplayPayment($params);
	}


	/**
	 * Affichage du mode de paiement Comnpay
	 */
	public function hookDisplayPayment($params)
	{
		if (!$this->isAvailable())
			return;

		global $smarty;

		$smarty->assign(
		    array(
		        'path_img' => $this->_path,
		        'this_path' => $this->_path,
				'this_path_ssl' => Tools::getShopDomainSsl(true, true).__PS_BASE_URI__.'modules/'.$this->name.'/'
		    )
		);
		return $this->display(__FILE__, '/views/templates/front/payment.tpl');
	}


	/**
	 * Traitement du retour de l'utilisateur après le paiement
	 */
	public function hookDisplayPaymentReturn($params)
	{
		if (!$this->isAvailable())
			return;
		
		// Get the result
		$result = Tools::getValue('result');
		
		// Specify template
		if ($result == "OK")
		{
			$template = 'authorised.tpl';
		}
		else
		{
			$template = 'error.tpl';
		}

		return $this->display(__FILE__, 'views/templates/front/result/'.$template);
	}

	/**
	 * Ajout d'une CSS personnalisée
	 */
	public function hookDisplayHeader()
	{
		Tools::addCSS(_THEME_CSS_DIR_.'css/comnpay.css');
	}

	/**
	 * Ajout d'une CSS personnalisée dans l'admin pour la page de configuration Comnpay
	 */
	public function hookDisplayBackOfficeHeader()
	{
		Tools::addCSS(_THEME_CSS_DIR_.'css/comnpay_back.css');
	} 


	public function isAvailable()
	{
		if (!$this->active)
			return false;

		if ( (Configuration::get('COMNPAY_GATEWAY_TPE_NUMBER') != "") &&
				(Configuration::get('COMNPAY_GATEWAY_SECRET_KEY') != "") )
			return true;

		return false;
	}


	/*
	 * Secret Validation
	 */
	public function validSec($values, $secret_key)
	{
	    if (isset($values['sec']) && $values['sec'] != "")
	    {
	        $sec = $values['sec'];
	        unset($values['sec']);
	        unset($values['cluster']);	 // Rajouté par les serveurs OVH et fausse la validation
	        return strtoupper(hash("sha512", base64_encode(implode("|",$values)."|".$secret_key))) == strtoupper($sec);
	    }
	    else
	    {
	        return false;
	    }
	}

	/**
	 * Changer Id Order State
	 */
	public function changeIdOrderState($transactionId, $stateId)
	{
		if ($transactionId=="")
			return false;

	    $orderHistory = new OrderHistory();
	    $orderHistory->id_order = $transactionId;
	    $orderHistory->changeIdOrderState($stateId, $transactionId);
	    $orderHistory->addWithemail();
	    $orderHistory->save();

	    return true;
	}


	public function preparePayment()
	{
		$context = $this->context;
		$cart = $context->cart;
		$customer = new Customer((int)$cart->id_customer);

		// On transforme le panier en commande
		$this->validateOrder($cart->id, Configuration::get('COMNPAY_OS_PENDING'), (float)$cart->getOrderTotal(true, Cart::BOTH), $this->displayName, NULL, array(), (int)$cart->id_currency, false, $customer->secure_key);

		// On prépare le formulaire qui enverra l'utilisateur sur la passerelle de paiement		
		$comnpay['montant'] = number_format( $cart->getOrderTotal(), 2 , '.', '');
		$comnpay['idTPE'] = Configuration::get('COMNPAY_GATEWAY_TPE_NUMBER');
		$comnpay['idTransaction'] = (int)$this->currentOrder;
		$comnpay['devise'] = $context->currency->iso_code;
		$comnpay['lang'] = Language::getIsoById($this->context->language->id);
		$comnpay['nom_produit'] = Configuration::get('PS_SHOP_NAME');
		$comnpay['source'] = $_SERVER['SERVER_NAME'];
		$comnpay['urlRetourOK'] = (Configuration::get('PS_SSL_ENABLED') ? 'https://' : 'http://').htmlspecialchars($_SERVER['HTTP_HOST'], ENT_COMPAT, 'UTF-8').__PS_BASE_URI__.'index.php?controller=order-confirmation&key='.$customer->secure_key.'&id_cart='.(int)$cart->id.'&id_module='.(int)$this->id.'&id_order='.(int)$this->currentOrder;
		$comnpay['urlRetourNOK'] = (Configuration::get('PS_SSL_ENABLED') ? 'https://' : 'http://').htmlspecialchars($_SERVER['HTTP_HOST'], ENT_COMPAT, 'UTF-8').__PS_BASE_URI__.'index.php?controller=order-confirmation&key='.$customer->secure_key.'&id_cart='.(int)$cart->id.'&id_module='.(int)$this->id.'&id_order='.(int)$this->currentOrder;
		$comnpay['urlIPN'] = (Configuration::get('PS_SSL_ENABLED') ? 'https' : 'http').'://'.$_SERVER['HTTP_HOST'].__PS_BASE_URI__.'modules/'.$this->name.'/ipn.php';
		$comnpay['extension'] = "prestashop-".$this->name."-".$this->version;
		$comnpay['key'] = Configuration::get('COMNPAY_GATEWAY_SECRET_KEY');

		// Encoding
		$comnpayWithKey = base64_encode(implode("|", $comnpay));
		unset($comnpay['key']);
		$comnpay['sec'] = hash("sha512",$comnpayWithKey);

		if (Configuration::get('COMNPAY_GATEWAY_CONFIG')=="HOMOLOGATION")
			$form_action = Configuration::get('COMNPAY_GATEWAY_HOMOLOGATION');
		else
			$form_action = Configuration::get('COMNPAY_GATEWAY_PRODUCTION');

		// Generate Form
		$form = "<form name=\"comnpay_form\" id=\"comnpay_form\" method=\"post\" action=\"".$form_action."\">";
		foreach ($comnpay as $key => $value) {
			$form .= '<input type="hidden" name="'.$key.'" value="'.$value.'"/>';
		}
		$form .= "</form>";

		global $smarty;
		$smarty->assign(array (
				'comnpayForm' => $form
			));
	}



	/**
	 * Administration
	 */
	public function getContent()
	{
		if (!isset($this->_html) || empty($this->_html))
			$this->_html = '';
		

		$msg_confirmation="";
		if (Tools::isSubmit('submitComnpay'))
		{
			Configuration::updateValue('COMNPAY_GATEWAY_CONFIG', Tools::getValue('COMNPAY_GATEWAY_CONFIG'));
			Configuration::updateValue('COMNPAY_GATEWAY_HOMOLOGATION', Tools::getValue('COMNPAY_GATEWAY_HOMOLOGATION'));
			Configuration::updateValue('COMNPAY_GATEWAY_PRODUCTION', Tools::getValue('COMNPAY_GATEWAY_PRODUCTION'));
			Configuration::updateValue('COMNPAY_GATEWAY_TPE_NUMBER', Tools::getValue('COMNPAY_GATEWAY_TPE_NUMBER'));
			Configuration::updateValue('COMNPAY_GATEWAY_SECRET_KEY', Tools::getValue('COMNPAY_GATEWAY_SECRET_KEY'));

			$msg_confirmation="<div class=\"conf\">Configuration sauvegardée</div>";
		}
		
		return $this->_html.'
		<div id="comnpay_configuration">
			<fieldset><legend><img src="../modules/'.$this->name.'/img/logo.gif" /> '.$this->l('Présentation').'</legend>
				<p>
					<img src="../modules/'.$this->name.'/img/comnpay.png" alt="Logo ComNpay" class="float_left" />
					<h3>'.$this->l('Accepts payments by credit cards with comNpay.', 'comnpay').'</h3>'.'
				</p>
			
			</fieldset>

			<div class="clear">&nbsp;</div>
			'.$msg_confirmation.'
			<form action="'.Tools::htmlentitiesUTF8($_SERVER['REQUEST_URI']).'" method="post">
				<fieldset><legend><img src="../img/admin/contact.gif" /> '.$this->l('Settings').'</legend>
					<div>
						<label for="tpe_number">'.$this->l('TPE Number').'</label>
						<div class="margin-form">
							<input type="text" id="tpe_number" size="30" name="COMNPAY_GATEWAY_TPE_NUMBER" value="'.Tools::safeOutput(Tools::getValue('COMNPAY_GATEWAY_TPE_NUMBER', Configuration::get('COMNPAY_GATEWAY_TPE_NUMBER'))).'" />
						</div>
						<div class="clear">&nbsp;</div>
						
						<label for="secret_key">'.$this->l('Secret Key').'</label>
						<div class="margin-form">
							<input type="text" id="secret_key" size="30" name="COMNPAY_GATEWAY_SECRET_KEY" value="'.Tools::safeOutput(Tools::getValue('COMNPAY_GATEWAY_SECRET_KEY', Configuration::get('COMNPAY_GATEWAY_SECRET_KEY'))).'" />
						</div>
						<div class="clear">&nbsp;</div>
						
						<label for="url_homologation">'.$this->l('URL homologation').'</label>
						<div class="margin-form">
							<input type="text" id="url_homologation" size="50" name="COMNPAY_GATEWAY_HOMOLOGATION" value="'.Tools::safeOutput(Tools::getValue('COMNPAY_GATEWAY_HOMOLOGATION', Configuration::get('COMNPAY_GATEWAY_HOMOLOGATION'))).'" />
						</div>
						<div class="clear">&nbsp;</div>

						<label for="url_production">'.$this->l('URL production').'</label>
						<div class="margin-form">
							<input type="text" id="url_production" size="50" name="COMNPAY_GATEWAY_PRODUCTION" value="'.Tools::safeOutput(Tools::getValue('COMNPAY_GATEWAY_PRODUCTION', Configuration::get('COMNPAY_GATEWAY_PRODUCTION'))).'" />
						</div>
						<div class="clear">&nbsp;</div>
						
						<label>'.$this->l('Plateforme').'</label>
						<div class="margin-form">
							<div class="plateforme">
								<input type="radio" id="homologation" name="COMNPAY_GATEWAY_CONFIG" value="HOMOLOGATION" class="choix_plateforme"
								'.((Tools::getValue('COMNPAY_GATEWAY_CONFIG', Configuration::get('COMNPAY_GATEWAY_CONFIG')) == "HOMOLOGATION") ? 'checked="checked"' : '').' />
								<label for="homologation" class="label_plateforme homologation">'.$this->l('Homologation').'</label>&nbsp;
							</div>
							<div class="clear">&nbsp;</div>
							<div class="plateforme">
								<input type="radio" id="production" name="COMNPAY_GATEWAY_CONFIG" value="PRODUCTION" class="choix_plateforme" 
								'.((Tools::getValue('COMNPAY_GATEWAY_CONFIG', Configuration::get('COMNPAY_GATEWAY_CONFIG')) == "PRODUCTION") ? 'checked="checked"' : '').' />
								<label for="production" class="label_plateforme production">'.$this->l('Production').'</label>
							</div>
						</div>
						<div class="clear">&nbsp;</div>
						
						<input type="submit" name="submitComnpay" value="'.$this->l('Update settings').'" class="button" />
					</div>
				</fieldset>
			</form>
			<div class="clear">&nbsp;</div>
		</div>';
	}

}
?>